<?php
/**
 * Page callbacks Information about the status of the system
 */

function testInfo($keyword,$file) {    // Test keyword from data file, returning true if found
  exec( 'writeup -qq -x ' . $keyword . ' ' . $file, $output, $return_var );
  return ($return_var == 0);
}

function getInfo($keyword,$file) {    // Get keyword from data file
  if (testInfo($keyword,$file)) $ret = exec( 'writeup -qq -x ' . $keyword . ' ' . $file );
  else $ret = '-';
  return $ret;
}

function getBuild($file) {    // Get build number of a document. If no SVN, use date
  $build = getInfo('SVNBUILD',$file);
  if ($build == '-') {
    $date = getInfo('DATE',$file );
    $build = substr($date,0,4) . substr($date,5,2) . substr($date,8,2);
  }
  return $build;
}

function getNodeBuild($nid) {    // Get build number of Drupal node
  $sql="SELECT log FROM {node} JOIN {node_revisions} ON {node}.vid = {node_revisions}.vid WHERE {node}.nid=%d;"; // find 
  return db_result(db_query($sql,$nid));
}
// generates a list of all files in the chosen directory and all subdirectories of specific extentions $allowext
function scanDirectories($rootDir, $allowext, &$allData = array()) {
    $dirContent = scandir($rootDir);
    foreach($dirContent as $key => $content) {
        if ($content == '.' || $content == '..') continue;
        $path = $rootDir . DIRECTORY_SEPARATOR . $content;
        $ext = substr($content, strrpos($content, '.') + 1);
        if(in_array($ext, $allowext) && is_file($path) && is_readable($path)) {
            $allData[] = $path;
        }
        else if(is_dir($path) && is_readable($path)) {
            // recursive callback to open new directory
            scanDirectories($path, $allowext, $allData);
        }
        // else unreadable garbage
    }
    return $allData;
}

function writeup_import_info_page() {
  $dir    = 'pub';
  $files = scanDirectories($dir, array("txt"));
//  $files = scandir($dir);
  //$foundfiles= var_export($files,TRUE);
  //$page .= '<pre>' . $foundfiles . '</pre>';
  $page = '<h2>List of files in SVN system</h2>';
  $page .='<table><tr><th>Filename</th><th>nid</th><th>File ver</th><th>Node ver</th><th>Title</th><tr>';
  foreach ($files as $file) {
    if (substr($file,-4) != '.txt') continue;
    $nid = getNid($file);
    $page .= '<tr><td>' . $file;
    $page .= '</td><td>' . $nid;
    $page .= '</td><td>' . getBuild($file);
    $page .= '</td><td>' . getNodeBuild($nid);
    $page .= '</td><td>' . strtotime(getInfo('SVNDATE',$file) . ' ' . getInfo('SVNTIME',$file));
    $page .= '</td><td>' . getInfo('TITLE',$file );
    $page .= '</td></tr>';
  }
  $page .= '</table>';
  return t($page);
}

function getNid($filename) { // returns node id or -1 if not found
  $sql="SELECT src FROM {url_alias} WHERE dst='%s';"; // get id of node
  $src = db_result(db_query($sql,substr($filename,strlen('pub/'),-4)));
  if ($src == FALSE) return -1;
  else return (int)substr($src,5);
}

function writeup_import_test_page() {
  $tim  = getdate(time());
  $mins = $tim[hours]*60+$tim[minutes];
  $page = '<h2> ' . writeup_import_help('admin/help#writeup_import',0) . '</h2>
  <ul><li>' . writeup_import_version() . '</li>
  <ul><li>writeup ' . exec( 'writeup --version' ) . '</li>
  <li>Current server day/time-in-mins : <strong>'
  . strval(date(w)) . '/' . strval($mins) . '</strong></li>';
  $page .= '<li>absolute seconds [time()]: <b>' . time() . '</b></li>';
  return t($page);
}

function writeup_import_update_node($filename) { // make changes to node using node_load and node_save --slow but accurate
  $log = '<tr><td>' . $filename . '</td>';
  $title = getInfo('TITLE',$filename );
  $nid = getNid($filename);
  if ($nid == -1) $nid = '-';
  else {
    $node = node_load($nid);
    if ($node == FALSE ) $title = 'Invalid node';
  }
  $log .= t('<td>@nid</td><td>@title</td>',array('@nid' => $nid, '@title' => $title));
  if (($nid != '-') && ($node != FALSE)) {
    $node->title = $title;
    $node->format = 2;
    $node->teaser = '';
    if (testInfo('publish',$filename)) $node->status  = getInfo('publish',$filename);
    if (testInfo('promote',$filename)) $node->promote = getInfo('promote',$filename);
    if (testInfo('sticky',$filename))  $node->sticky  = getInfo('sticky',$filename);
    // process body and put it into database
    exec( 'writeup -bqq -s drupal=true ' . $filename);
    $node->body = file_get_contents(substr($filename,0,-4) . '.html');
    $node->log = getBuild($filename); // put version/build number in log
    $svntimestamp=strtotime(getInfo('SVNDATE',$filename) . ' ' . getInfo('SVNTIME',$filename));
    $node = node_submit($node);
//  $log .= '<pre>' . print_r($node,TRUE) . '</pre>';
    //$node->created = $svntimestamp;
    //$node->changed = $svntimestamp;
    //$node->revision_timestamp = $svntimestamp;
    node_save($node);
    $sql="UPDATE {node} SET changed=%d, created=%d WHERE nid=%d;"; // have to manually set timestamps
    db_query($sql,$svntimestamp,$svntimestamp,$nid);
//$log .='<td>' . $svntimestamp
  }
  return $log;
}
function writeup_import_update_db($filename) { // make changes to node using direct database writes -- fast but misses things
  $log = '<tr><td>' . $filename . '</td>';
  $title = getInfo('TITLE',$filename );
  $nid = getNid($filename);
  if ($nid == -1) $nid = '-';
  $log .= t('<td>@nid</td><td>@title</td>',array('@nid' => $nid, '@title' => $title));
  if ($nid != '-') {
    $sql="SELECT vid FROM {node} WHERE nid=%d;"; // get number of revision
    $vid = db_result(db_query($sql,$nid));
    
    // process title and other node attributes and put it into database
    $sql = "UPDATE {node} SET title='%s'";
    $sqlFields = array($title);
    if (testInfo('publish',$filename)) {
      $sql .= ", status=%d";
      $sqlFields[] = getInfo('publish',$filename);
    }
    if (testInfo('promote',$filename)) {
      $sql .= ", promote=%d";
      $sqlFields[] = getInfo('promote',$filename);
    }
    if (testInfo('sticky',$filename)) {
      $sql .= ", sticky=%d";
      $sqlFields[] = getInfo('sticky',$filename);
    }
    $sql .= " WHERE nid=%d;";
    $sqlFields[] = $nid;
    db_query($sql,$sqlFields);
    // process body and put it into database
    exec( 'writeup -bqq -s drupal=true ' . $filename);
    $newcontent = file_get_contents(substr($filename,0,-4) . '.html');
    $sql="UPDATE {node_revisions} SET title='%s', body='%s', teaser='', format=2, log='%s' WHERE vid=%d;"; // format 2=full HTML
    //$log .= '<br />SQL=' . $sql;
    db_query($sql,$title,$newcontent,getBuild($file),$vid);
  }
  return $log;
}
function writeup_import_update_page() {
  $page = '<h2>Updating content from SVN</h2>';
//  $lastline = system( 'pwd');
//  $page .= $lastline . '<br />';
  $lastline = exec( 'svn up --config-dir /var/www/html pub'); //may need to set this up if password needed
  $page .= '<pre>' . $lastline  . '</pre><hr />';

  $dir    = 'pub';  //find all the files
  $files = scanDirectories($dir, array("txt"));
//  $files = scandir($dir);
  $page .= '<table><tr><th>File</th><th>nid</th><th>Result</th></tr>';
  foreach ($files as $file) {
    if ( (substr($file,-4) != '.txt') || ($file == 'top.txt') || ($file == 'bot.txt')) continue;
    $page .= writeup_import_update_node($file);
  }
  $page .= '</table>';
  return t($page);
}
function writeup_import_dumpnode($nid = '-1') {
  $page = '<h2>Dumping contents of node ' . $nid . '</h2>';
  if ($nid == -1) return t('No node specified');
  $node = node_load($nid);
  if ($node == FALSE ) return t('Invalid node specified');
  $page .= 'nid: ' . $node->nid . '<br />';
  $page .= 'title: ' . $node->title . '<br />';
  $page .= '<pre>' . print_r($node,TRUE) . '</pre>';
  return t($page);
}
